// voltage.c
//
// Code to monitor input voltage
//
// (c) Bill Dally, April 2020
//
// Copyright 2020 Bill Dally
// 
// Licensed under the Ventilator Open Source License (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// ./LICENSE
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// 1023 is 3.3V after 33k/10k divider - 1/4.3 - so 1023 is 14.19
// each count is 13.87mV

// want count * 13.87 / 100
// do count * (13.87 / 100) * 64k and shift 16

#include <avr/io.h>
#include <ctype.h>
#include "adc.h"

#define V12CONV 9090


// return input voltage in 100s of mV
uint16_t get_in_v(void) {
  uint16_t raw ;
  uint32_t vlong ;
  uint16_t vout ;
  raw = get_adc(V12_ADC_CHAN) ;
  vlong = ((uint32_t) raw * 9090) ;
  vout = vlong >> 16 ;
  return(vout) ;
}

